#pragma once
#ifndef AUTOMATA_H
#define AUTOMATA_H

#include "SDL.h"

//Forward declarations
class Synth;

/*
Generative Music using Cellular Automata
Tech Demo

by Brian MacIntosh

V0.5: 12/10/12

You are free to modify and distribute this code or binaries for commercial
or non-commercial purposes, provided this header remains intact.
No warranty of fitness for any purpose, express or implied, is given.
*/

/*
Represents a 1-D cellular automata over time
*/

class Automata
{
public:
	Automata(int width, int depth);

	//Set bandwidth that actually maps to notes
	void SetEffectiveWidth(int width);

	//Set the rule to use
	void SetRule(char rule);

	//Set the chance for a line to be randomized
	void SetRandomlineChance(int chance);

	//Set the chance for the rule to be randomized
	void SetRandomruleChance(int chance);

	//Manually trigger a line respawn event
	void RandomLine();

	//Set the frequency of a given column
	void SetFreq(int index, double freq);

	//Spawn the next iteration
	void Cycle();

	//Render the sound
	//DEPRECATED: Use Synths instead
	void RenderSines(Uint8* stream, int len, SDL_AudioSpec audioFormat) const;

	//Render the sound to a synth
	void RenderSynth(Synth* synth, int duration) const;

	//Render the cell grid
	void RenderCells(int left, int top, SDL_Surface* tex, SDL_Surface* surface) const;

	//Read out the value of this automata in binary
	int ParseValue();

	//Read out the value of the highest-order bit in this automata
	int HighBit();

protected:
	int bufferWidth;
	int bufferDepth;
	int effectiveBufferWidth;

	double* freqMap;
	double* freqPeriod;
	int* volMap;

	bool** cellBuffer;

	int randomlineChance;
	int randomruleChance;

	int rule;
};

#endif