#include "Synth.h"
#include <math.h>
#include <cstdlib>


Synth::Tone::Tone(double freq, int dur)
{
	frequency = freq;
	duration = dur;
	period = 0;
}

bool Synth::Tone::Ended(SDL_AudioSpec audioFormat, int r) const
{
	return period >= audioFormat.freq * (duration + r) / 1000.0;
}


Synth::Synth()
{
	a = 0;
	d = 0;
	s = 1;
	r = 1000;
}


//Render and advance the sound from this synth
void Synth::Render(Uint8* stream, int len, SDL_AudioSpec audioFormat)
{
	bool ended = false;
	for (auto it = tones.begin(); it != tones.end(); it++)
	{
		if (!it->Ended(audioFormat, r))
		{
			Uint8* sine = (Uint8*)calloc(len, sizeof(Uint8));

			ended = false;
			double vol = 127;
			for (int d = 0; d < len; d++)
			{
				if (ended)
				{
					vol -= 1 / (r / 1000.0 * audioFormat.freq);
					if (vol < 0) vol = 0;
				}

				it->period++;
				double v1 = sin(2 * 3.1416 * it->frequency / audioFormat.freq * it->period);
				if (v1 > 0)
					v1 = (int)(v1 + 0.5);
				else
					v1 = (int)(v1 - 0.5);
				sine[d] = vol * (1 + v1);

				//Shut down on 0
				/*if (d > 0
					&& !cellBuffer[bufferDepth - 1][c]
					&& ((sine[d] <= 127 && sine[d-1] >= 127) || (sine[d] >= 127 && sine[d-1] <= 127)))
				{
					sine[d] = 0;
					freqPeriod[c] = 0;
					break;
				}*/

				//End tone
				if (it->Ended(audioFormat, r))
					ended = true;
			}
			SDL_MixAudio(stream, sine, len, SDL_MIX_MAXVOLUME / tones.size());
			free(sine);
		}
	}

	//Remove expired tones
	//HACK: assumes all tones are same length
	for (auto it = tones.begin(); it != tones.end(); it++)
		if (it->Ended(audioFormat, r))
		{
			tones.pop_front();
			break;
		}
}


//Play a tone with this synth
void Synth::PlayTone(double freq, int dur)
{
	tones.push_back(Tone(freq, dur));
}


//Set the waveform used by this synth
void Synth::SetWaveform()
{

}


//Set the ADSR values of this synth, in ms
void Synth::SetADSR(int a, int d, double s, int r)
{
	this->a = a;
	this->d = d;
	this->s = s;
	this->r = r;
}